@@ -1,6 +1,7 @@
-from .compat import threading
 import weakref
 
+from .compat import threading
+
 
 class NameRegistry(object):
     """Generates and return an object, keeping it as a
@@ -37,6 +38,7 @@ class NameRegistry(object):
      method.
 
     """
+
     _locks = weakref.WeakValueDictionary()
     _mutex = threading.RLock()
 
@@ -76,11 +78,13 @@ class NameRegistry(object):
                     return self._values[identifier]
                 else:
                     self._values[identifier] = value = self.creator(
-                        identifier, *args, **kw)
+                        identifier, *args, **kw
+                    )
                     return value
             except KeyError:
                 self._values[identifier] = value = self.creator(
-                    identifier, *args, **kw)
+                    identifier, *args, **kw
+                )
                 return value
         finally:
             self._mutex.release()
